@@ -1,6 +1,10 @@
-import types, sys, functools
+import functools
+import sys
+import types
+
 from . import termcap
 
+
 def eval_when(when):
     if isinstance(when, file) or \
       when in ('always', 'never', 'auto', sys.stderr, sys.stdout):
