@@ -1,13 +1,24 @@
 # -*- coding: utf-8 -*-
-from .buffer import Buffer
+import logging
+import re
+import string
+import subprocess
+import sys
+import threading
+import time
+
+from .. import atexit
+from .. import term
+from ..context import context
+from ..log import Logger
 from ..timeout import Timeout
-from .. import context, term, atexit
-from ..util import misc, fiddling
-import re, threading, sys, time, subprocess, logging, string
+from ..util import fiddling
+from ..util import misc
+from ..util import packing
+from .buffer import Buffer
 
-log = logging.getLogger(__name__)
 
-class tube(Timeout):
+class tube(Timeout, Logger):
     """
     Container of all the tube functions common to sockets, TTYs and SSH connetions.
     """
@@ -19,8 +30,13 @@ class tube(Timeout):
     #: and related functions.
     newline = '\n'
 
-    def __init__(self, timeout = default):
+    def __init__(self, timeout = default, level = None):
         super(tube, self).__init__(timeout)
+
+        Logger.__init__(self, None)
+        if level is not None:
+            self.setLevel(level)
+
         self.buffer          = Buffer()
         atexit.register(self.close)
 
@@ -108,14 +124,16 @@ class tube(Timeout):
         with self.local(timeout):
             data = self.recv_raw(4096)
 
-        if data and log.isEnabledFor(logging.DEBUG):
-            log.debug('Received %#x bytes:' % len(data))
+        if data and self.isEnabledFor(logging.DEBUG):
+            self.debug('Received %#x bytes:' % len(data))
 
-            if all(c in string.printable for c in data):
+            if len(set(data)) == 1 and len(data) > 1:
+                self.indented('%r * %#x' % (data[0], len(data)), level = logging.DEBUG)
+            elif all(c in string.printable for c in data):
                 for line in data.splitlines(True):
-                    log.indented(repr(line), level = logging.DEBUG)
+                    self.indented(repr(line), level = logging.DEBUG)
             else:
-                log.indented(fiddling.hexdump(data), level = logging.DEBUG)
+                self.indented(fiddling.hexdump(data), level = logging.DEBUG)
 
         if data:
             self.buffer.add(data)
@@ -126,7 +144,7 @@ class tube(Timeout):
     def _recv(self, numb = 4096, timeout = default):
         """_recv(numb = 4096, timeout = default) -> str
 
-        Recieves one chunk of from the internal buffer or from the OS if the
+        Receives one chunk of from the internal buffer or from the OS if the
         buffer is empty.
         """
         data = ''
@@ -165,7 +183,7 @@ class tube(Timeout):
             while not pred(data):
                 try:
                     res = self.recv(1)
-                except:
+                except Exception:
                     self.unrecv(data)
                     return ''
 
@@ -180,7 +198,7 @@ class tube(Timeout):
     def recvn(self, numb, timeout = default):
         """recvn(numb, timeout = default) -> str
 
-        Recieves exactly `n` bytes.
+        Receives exactly `n` bytes.
 
         If the request is not satisfied before ``timeout`` seconds pass,
         all data is buffered and an empty string (``''``) is returned.
@@ -210,8 +228,8 @@ class tube(Timeout):
                 >>> t.recv_raw = lambda *a: time.sleep(0.01) or 'a'
                 >>> t.recvn(10, timeout=0.05)
                 ''
-                >>> t.recvn(10, timeout=0.05)
-                'aaaaaaaaaa'
+                >>> t.recvn(10, timeout=0.06) # doctest: +ELLIPSIS
+                'aaaaaa...'
         """
         # Keep track of how much data has been received
         # It will be pasted together at the end if a
@@ -228,7 +246,7 @@ class tube(Timeout):
     def recvuntil(self, delims, drop=False, timeout = default):
         """recvuntil(delims, timeout = default) -> str
 
-        Recieve data until one of `delims` is encountered.
+        Receive data until one of `delims` is encountered.
 
         If the request is not satisfied before ``timeout`` seconds pass,
         all data is buffered and an empty string (``''``) is returned.
@@ -287,7 +305,7 @@ class tube(Timeout):
             while self.countdown_active():
                 try:
                     res = self.recv(timeout=self.timeout)
-                except:
+                except Exception:
                     self.unrecv(''.join(data) + top)
                     raise
 
@@ -316,10 +334,10 @@ class tube(Timeout):
 
         return ''
 
-    def recvlines(self, numlines, keepends = False, timeout = default):
+    def recvlines(self, numlines=2**20, keepends = False, timeout = default):
         r"""recvlines(numlines, keepends = False, timeout = default) -> str list
 
-        Recieve up to ``numlines`` lines.
+        Receive up to ``numlines`` lines.
 
         A "line" is any sequence of bytes terminated by the byte sequence
         set by :attr:`newline`, which defaults to ``'\n'``.
@@ -361,7 +379,7 @@ class tube(Timeout):
                     # restore the original, unmodified data to the buffer
                     # in the event of a timeout.
                     res = self.recvline(keepends=True, timeout=timeout)
-                except:
+                except Exception:
                     self.unrecv(''.join(lines))
                     raise
 
@@ -444,7 +462,7 @@ class tube(Timeout):
             while self.countdown_active():
                 try:
                     line = self.recvline(keepends=True)
-                except:
+                except Exception:
                     self.buffer.add(tmpbuf)
                     raise
 
@@ -498,8 +516,8 @@ class tube(Timeout):
     def recvline_startswith(self, delims, keepends = False, timeout = default):
         r"""recvline_startswith(delims, keepends = False, timeout = default) -> str
 
-        Keep recieving lines until one is found that starts with one of
-        `delims`.  Returns the last line recieved.
+        Keep receiving lines until one is found that starts with one of
+        `delims`.  Returns the last line received.
 
         If the request is not satisfied before ``timeout`` seconds pass,
         all data is buffered and an empty string (``''``) is returned.
@@ -536,8 +554,8 @@ class tube(Timeout):
     def recvline_endswith(self, delims, keepends = False, timeout = default):
         r"""recvline_endswith(delims, keepends = False, timeout = default) -> str
 
-        Keep recieving lines until one is found that starts with one of
-        `delims`.  Returns the last line recieved.
+        Keep receiving lines until one is found that starts with one of
+        `delims`.  Returns the last line received.
 
         If the request is not satisfied before ``timeout`` seconds pass,
         all data is buffered and an empty string (``''``) is returned.
@@ -645,15 +663,15 @@ class tube(Timeout):
 
         return self.buffer.get()
 
-    def recvall(self):
+    def recvall(self, timeout=Timeout.forever):
         """recvall() -> str
 
         Receives data until EOF is reached.
         """
 
-        with log.waitfor('Recieving all data') as h:
+        with self.waitfor('Receiving all data') as h:
             l = len(self.buffer)
-            with self.local(Timeout.forever):
+            with self.local(timeout):
                 try:
                     while True:
                         l = misc.size(len(self.buffer))
@@ -687,16 +705,18 @@ class tube(Timeout):
             'hello'
         """
 
-        if log.isEnabledFor(logging.DEBUG):
-            log.debug('Sent %#x bytes:' % len(data))
-            if all(c in string.printable for c in data):
+        if self.isEnabledFor(logging.DEBUG):
+            self.debug('Sent %#x bytes:' % len(data))
+            if len(set(data)) == 1:
+                self.indented('%r * %#x' % (data[0], len(data)))
+            elif all(c in string.printable for c in data):
                 for line in data.splitlines(True):
-                    log.indented(repr(line), level = logging.DEBUG)
+                    self.indented(repr(line), level = logging.DEBUG)
             else:
-                log.indented(fiddling.hexdump(data), level = logging.DEBUG)
+                self.indented(fiddling.hexdump(data), level = logging.DEBUG)
         self.send_raw(data)
 
-    def sendline(self, line):
+    def sendline(self, line=''):
         r"""sendline(data)
 
         Shorthand for ``t.send(data + t.newline)``.
@@ -715,6 +735,10 @@ class tube(Timeout):
 
         self.send(line + self.newline)
 
+    def sendlines(self, lines=[]):
+        for line in lines:
+            self.sendline(line)
+
     def sendafter(self, delim, data, timeout = default):
         """sendafter(delim, data, timeout = default) -> str
 
@@ -761,18 +785,19 @@ class tube(Timeout):
         Thus it only works in while in :data:`pwnlib.term.term_mode`.
         """
 
-        log.info('Switching to interactive mode')
+        self.info('Switching to interactive mode')
 
         go = threading.Event()
         def recv_thread():
             while not go.isSet():
                 try:
                     cur = self.recv(timeout = 0.05)
+                    cur = cur.replace('\r\n', '\n')
                     if cur:
                         sys.stdout.write(cur)
                         sys.stdout.flush()
                 except EOFError:
-                    log.info('Got EOF while reading in interactive')
+                    self.info('Got EOF while reading in interactive')
                     break
 
         t = context.Thread(target = recv_thread)
@@ -791,11 +816,11 @@ class tube(Timeout):
                         self.send(data)
                     except EOFError:
                         go.set()
-                        log.info('Got EOF while sending in interactive')
+                        self.info('Got EOF while sending in interactive')
                 else:
                     go.set()
         except KeyboardInterrupt:
-            log.info('Interrupted')
+            self.info('Interrupted')
             go.set()
 
         while t.is_alive():
@@ -833,8 +858,8 @@ class tube(Timeout):
     def clean_and_log(self, timeout = 0.05):
         r"""clean_and_log(timeout = 0.05)
 
-        Works exactly as :meth:`pwnlib.tubes.tube.tube.clean`, but logs recieved
-        data with :meth:`pwnlib.log.info`.
+        Works exactly as :meth:`pwnlib.tubes.tube.tube.clean`, but logs received
+        data with :meth:`pwnlib.self.info`.
 
         Returns:
 
@@ -850,20 +875,14 @@ class tube(Timeout):
             >>> t.fileno        = lambda: 1234
             >>> with context.local(log_level='info'):
             ...     data = t.clean_and_log() #doctest: +ELLIPSIS
+            [DEBUG] Received 0xb bytes:
                 'hooray_data'
             >>> data
             'hooray_data'
             >>> context.clear()
         """
-        data = self.clean(timeout)
-
-        if all(c in string.printable for c in data):
-            for line in data.splitlines(True):
-                log.indented(repr(line))
-        else:
-            log.indented(fiddling.hexdump(data))
-
-        return data
+        with context.local(log_level='debug'):
+            return self.clean(timeout)
 
     def connect_input(self, other):
         """connect_input(other)
@@ -1018,6 +1037,8 @@ class tube(Timeout):
         while self.connected():
             time.sleep(0.05)
 
+    wait = wait_for_close
+
     def can_recv(self, timeout = 0):
         """can_recv(timeout = 0) -> bool
 
@@ -1076,7 +1097,7 @@ class tube(Timeout):
 
         Closes the tube for futher reading or writing depending on `direction`.
 
-        Args:
+        Arguments:
           direction(str): Which direction to close; "in", "read" or "recv"
             closes the tube in the ingoing direction, "out", "write" or "send"
             closes it in the outgoing direction.
@@ -1113,7 +1134,7 @@ class tube(Timeout):
 
         Returns True if the tube is connected in the specified direction.
 
-        Args:
+        Arguments:
           direction(str): Can be the string 'any', 'in', 'read', 'recv',
                           'out', 'write', 'send'.
 
@@ -1256,3 +1277,59 @@ class tube(Timeout):
         """
 
         raise NotImplementedError()
+
+    #: Alias for :meth:`recv`
+    def read(self, *a, **kw): return self.recv(*a, **kw)
+    #: Alias for :meth:`recvpred`
+    def readpred(self, *a, **kw): return self.recvpred(*a, **kw)
+    #: Alias for :meth:`recvn`
+    def readn(self, *a, **kw): return self.recvn(*a, **kw)
+    #: Alias for :meth:`recvuntil`
+    def readuntil(self, *a, **kw): return self.recvuntil(*a, **kw)
+    #: Alias for :meth:`recvlines`
+    def readlines(self, *a, **kw): return self.recvlines(*a, **kw)
+    #: Alias for :meth:`recvline`
+    def readline(self, *a, **kw): return self.recvline(*a, **kw)
+    #: Alias for :meth:`recvline_pred`
+    def readline_pred(self, *a, **kw): return self.recvline_pred(*a, **kw)
+    #: Alias for :meth:`recvline_contains`
+    def readline_contains(self, *a, **kw): return self.recvline_contains(*a, **kw)
+    #: Alias for :meth:`recvline_startswith`
+    def readline_startswith(self, *a, **kw): return self.recvline_startswith(*a, **kw)
+    #: Alias for :meth:`recvline_endswith`
+    def readline_endswith(self, *a, **kw): return self.recvline_endswith(*a, **kw)
+    #: Alias for :meth:`recvregex`
+    def readregex(self, *a, **kw): return self.recvregex(*a, **kw)
+    #: Alias for :meth:`recvline_regex`
+    def readline_regex(self, *a, **kw): return self.recvline_regex(*a, **kw)
+    #: Alias for :meth:`recvrepeat`
+    def readrepeat(self, *a, **kw): return self.recvrepeat(*a, **kw)
+    #: Alias for :meth:`recvall`
+    def readall(self, *a, **kw): return self.recvall(*a, **kw)
+
+    #: Alias for :meth:`send`
+    def write(self, *a, **kw): return self.send(*a, **kw)
+    #: Alias for :meth:`sendline`
+    def writeline(self, *a, **kw): return self.sendline(*a, **kw)
+    #: Alias for :meth:`sendafter`
+    def writeafter(self, *a, **kw): return self.sendafter(*a, **kw)
+    #: Alias for :meth:`sendlineafter`
+    def writelineafter(self, *a, **kw): return self.sendlineafter(*a, **kw)
+    #: Alias for :meth:`sendthen`
+    def writethen(self, *a, **kw): return self.sendthen(*a, **kw)
+    #: Alias for :meth:`sendlinethen`
+    def writelinethen(self, *a, **kw): return self.sendlinethen(*a, **kw)
+
+    def p64(self, *a, **kw):        return self.send(packing.p64(*a, **kw))
+    def p32(self, *a, **kw):        return self.send(packing.p32(*a, **kw))
+    def p16(self, *a, **kw):        return self.send(packing.p16(*a, **kw))
+    def p8(self, *a, **kw):         return self.send(packing.p8(*a, **kw))
+    def pack(self, *a, **kw):       return self.send(packing.pack(*a, **kw))
+
+    def u64(self, *a, **kw):        return packing.u64(self.recvn(8), *a, **kw)
+    def u32(self, *a, **kw):        return packing.u32(self.recvn(4), *a, **kw)
+    def u16(self, *a, **kw):        return packing.u16(self.recvn(2), *a, **kw)
+    def u8(self, *a, **kw):         return packing.u8(self.recvn(1), *a, **kw)
+    def unpack(self, *a, **kw):     return packing.unpack(self.recvn(context.bytes), *a, **kw)
+
+    def flat(self, *a, **kw):       return self.send(packing.flat(*a,**kw))
