@@ -1,6 +1,18 @@
-import socket, re, os, stat, errno, string, base64, logging
+import base64
+import errno
+import os
+import platform
+import re
+import socket
+import stat
+import string
+
 from . import lists
-log = logging.getLogger(__name__)
+from . import fiddling
+from ..context import context
+from ..log import getLogger
+
+log = getLogger(__name__)
 
 def align(alignment, x):
     """align(alignment, x) -> int
@@ -44,7 +56,7 @@ def size(n, abbriv = 'B', si = False):
 
     Convert the length of a bytestream to human readable form.
 
-    Args:
+    Arguments:
       n(int,str): The length to convert to human readable form
       abbriv(str):
 
@@ -74,28 +86,37 @@ def size(n, abbriv = 'B', si = False):
 
     return '%.02fP%s' % (n / base, abbriv)
 
+KB = 1024
+MB = 1024 * KB
+GB = 1024 * MB
 
-def read(path):
-    """read(path) -> str
+KiB = 1000
+MiB = 1000 * KB
+GiB = 1000 * MB
+
+def read(path, count=-1, skip=0):
+    r"""read(path, count=-1, skip=0) -> str
 
     Open file, return content.
 
     Examples:
-        >>> read('pwnlib/util/misc.py').split('\\n')[0]
-        'import socket, re, os, stat, errno, string, base64, logging'
+        >>> read('/proc/self/exe')[:4]
+        '\x7fELF'
     """
     path = os.path.expanduser(os.path.expandvars(path))
     with open(path) as fd:
-        return fd.read()
+        if skip:
+            fd.seek(skip)
+        return fd.read(count)
 
 
-def write(path, data = '', create_dir = False):
+def write(path, data = '', create_dir = False, mode = 'w'):
     """Create new file or truncate existing to zero length and write data."""
     path = os.path.expanduser(os.path.expandvars(path))
     if create_dir:
         path = os.path.realpath(path)
         mkdir_p(os.path.dirname(path))
-    with open(path, 'w') as f:
+    with open(path, mode) as f:
         f.write(data)
 
 def which(name, all = False):
@@ -107,7 +128,7 @@ def which(name, all = False):
     If `all` is :const:`True` the set of all found locations is returned, else
     the first occurence or :const:`None` is returned.
 
-    Args:
+    Arguments:
       `name` (str): The file to search for.
       `all` (bool):  Whether to return all locations where `name` was found.
 
@@ -119,6 +140,10 @@ def which(name, all = False):
       >>> which('sh')
       '/bin/sh'
 """
+    # If name is a path, do not attempt to resolve it.
+    if os.path.sep in name:
+        return name
+
     isroot = os.getuid() == 0
     out = set()
     try:
@@ -149,39 +174,58 @@ def run_in_new_terminal(command, terminal = None, args = None):
 
     Run a command in a new terminal.
 
-    If X11 is detected, the terminal will be launched with
-    ``x-terminal-emulator``.
+    When `terminal` is not set:
+      - If `context.terminal` is set it will be used.  If it is an iterable then
+        `context.terminal[1:]` are default arguments.
+      - If X11 is detected (by the presence of the ``DISPLAY`` environment
+        variable), ``x-terminal-emulator`` is used.
+      - If tmux is detected (by the presence of the ``TMUX`` environment
+        variable), a new pane will be opened.
 
-    If X11 is not detected, a new tmux pane is opened if possible.
-
-    Args:
+    Arguments:
       command (str): The command to run.
       terminal (str): Which terminal to use.
       args (list): Arguments to pass to the terminal
 
     Returns:
       None
+
     """
 
     if not terminal:
-        if 'XAUTHORITY' in os.environ:
+        if context.terminal:
+            terminal = context.terminal[0]
+            args     = context.terminal[1:]
+        elif 'DISPLAY' in os.environ:
             terminal = 'x-terminal-emulator'
             args     = ['-e']
-
         elif 'TMUX' in os.environ:
             terminal = 'tmux'
             args     = ['splitw']
 
     if not terminal:
-        log.error('could not find terminal: %s' % terminal)
+        log.error('Argument `terminal` is not set, and could not determine a default')
+
+    terminal_path = which(terminal)
+
+    if not terminal_path:
+        log.error('Could not find terminal: %s' % terminal)
+
+    argv = [terminal_path] + args
+
+    if isinstance(command, str):
+        argv += [command]
+    elif isinstance(command, (list, tuple)):
+        argv += list(command)
 
-    argv    = [which(terminal)] + args + [command]
     log.debug("Launching a new terminal: %r" % argv)
 
     if os.fork() == 0:
-        os.close(0)
-        os.close(1)
-        os.close(2)
+        # Closing the file descriptors makes everything fail under tmux on OSX.
+        if platform.system() != 'Darwin':
+            os.close(0)
+            os.close(1)
+            os.close(2)
         os.execv(argv[0], argv)
         os._exit(1)
 
@@ -190,7 +234,7 @@ def parse_ldd_output(output):
     Returns a dictionary of {path: address} for
     each library required by the specified binary.
 
-    Args:
+    Arguments:
       output(str): The output to parse
 
     Example:
@@ -247,9 +291,9 @@ def sh_string(s):
         >>> print sh_string("foo\\\\'bar")
         "foo\\\\'bar"
         >>> print sh_string("foo\\x01'bar")
-        "$( (echo Zm9vASdiYXI=|(base64 -d||openssl enc -d -base64)||echo -en 'foo\\x01\\x27bar') 2>/dev/null)"
-        >>> print subprocess.check_output("echo -n " + sh_string("foo\\\\'bar"), shell = True)
-        foo\\'bar
+        "$(printf 'foo\\001\\047bar')"
+        >>> print `subprocess.check_output("echo -n " + sh_string("foo\\x01'bar"), shell = True)`
+        "foo\\x01'bar"
     """
 
     very_good = set(string.ascii_letters + string.digits)
@@ -258,6 +302,8 @@ def sh_string(s):
 
     if '\x00' in s:
         log.error("sh_string(): Cannot create a null-byte")
+    if s.endswith('\n'):
+        log.error("sh_string(): Cannot create a newline-terminated string")
 
     if all(c in very_good for c in s):
         return s
@@ -276,21 +322,139 @@ def sh_string(s):
         for c in s:
             if c == '\\':
                 fixed += '\\\\'
+            elif c == '\n':
+                fixed += '\\n'
             elif c in good:
                 fixed += c
             else:
-                fixed += '\\x%02x' % ord(c)
-        return '"$( (echo %s|(base64 -d||openssl enc -d -base64)||echo -en \'%s\') 2>/dev/null)"' % (base64.b64encode(s), fixed)
+                fixed += '\\%03o' % ord(c)
+        return '"$(printf \'%s\')"' % fixed
+
+def sh_prepare(variables, export = False):
+    """Outputs a posix compliant shell command that will put the data specified
+    by the dictionary into the environment.
+
+    It is assumed that the keys in the dictionary are valid variable names that
+    does not need any escaping.
+
+    Arguments:
+      variables(dict): The variables to set.
+      export(bool): Should the variables be exported or only stored in the shell environment?
+      output(str): A valid posix shell command that will set the given variables.
+
+    It is assumed that `var` is a valid name for a variable in the shell.
+
+    Examples:
+
+        >>> print sh_prepare({'X': 'foobar'})
+        X=foobar
+        >>> r = sh_prepare({'X': 'foobar', 'Y': 'cookies'})
+        >>> r == 'X=foobar;Y=cookies' or r == 'Y=cookies;X=foobar'
+        True
+        >>> print sh_prepare({'X': 'foo bar'})
+        X='foo bar'
+        >>> print sh_prepare({'X': "foo'bar"})
+        X="foo'bar"
+        >>> print sh_prepare({'X': "foo\\\\bar"})
+        X='foo\\bar'
+        >>> print sh_prepare({'X': "foo\\\\'bar"})
+        X="foo\\\\'bar"
+        >>> print sh_prepare({'X': "foo\\x01'bar"})
+        X="$(printf 'foo\\001\\047bar')"
+        >>> print sh_prepare({'X': "foo\\x01'bar"}, export = True)
+        export X="$(printf 'foo\\001\\047bar')"
+        >>> print sh_prepare({'X': "foo\\x01'bar\\n"})
+        X="$(printf 'foo\\001\\047bar\\nx')";X=${X%x}
+        >>> print sh_prepare({'X': "foo\\x01'bar\\n"}, export = True)
+        X="$(printf 'foo\\001\\047bar\\nx')";export X=${X%x}
+        >>> print `subprocess.check_output('%s;echo -n "$X"' % sh_prepare({'X': "foo\\x01'bar"}), shell = True)`
+        "foo\\x01'bar"
+    """
+
+    out = []
+    export = 'export ' if export else ''
+
+    for k, v in variables.items():
+        if v.endswith('\n'):
+            out.append('%s=%s;%s%s=${%s%%x}' % (k, sh_string(v + "x"), export, k, k))
+        else:
+            out.append('%s%s=%s' % (export, k, sh_string(v)))
+    return ';'.join(out)
+
+def sh_command_with(f, *args):
+    """sh_command_with(f, arg0, ..., argN) -> command
+
+    Returns a command create by evaluating `f(new_arg0, ..., new_argN)`
+    whenever `f` is a function and `f % (new_arg0, ..., new_argN)` otherwise.
+
+    If the arguments are purely alphanumeric, then they are simply passed to
+    function. If they are simple to escape, they will be escaped and passed to
+    the function.
+
+    If the arguments contain trailing newlines, then it is hard to use them
+    directly because of a limitation in the posix shell. In this case the
+    output from `f` is prepended with a bit of code to create the variables.
+
+    Examples:
+
+        >>> print sh_command_with(lambda: "echo hello")
+        echo hello
+        >>> print sh_command_with(lambda x: "echo " + x, "hello")
+        echo hello
+        >>> print sh_command_with(lambda x: "echo " + x, "\\x01")
+        echo "$(printf '\\001')"
+        >>> import random
+        >>> random.seed(1)
+        >>> print sh_command_with(lambda x: "echo " + x, "\\x01\\n")
+        dwtgmlqu="$(printf '\\001\\nx')";dwtgmlqu=${dwtgmlqu%x};echo "$dwtgmlqu"
+        >>> random.seed(1)
+        >>> print sh_command_with("echo %s", "\\x01\\n")
+        dwtgmlqu="$(printf '\\001\\nx')";dwtgmlqu=${dwtgmlqu%x};echo "$dwtgmlqu"
+    """
+
+    args = list(args)
+    out = []
+
+    for n in range(len(args)):
+        if args[n].endswith('\n'):
+            v = fiddling.randoms(8)
+            out.append(sh_prepare({v: args[n]}))
+            args[n] = '"$%s"' % v
+        else:
+            args[n] = sh_string(args[n])
+    if hasattr(f, '__call__'):
+        out.append(f(*args))
+    else:
+        out.append(f % tuple(args))
+    return ';'.join(out)
+
+def dealarm_shell(tube):
+    """Given a tube which is a shell, dealarm it.
+    """
+    tube.clean()
+
+    tube.sendline('which python || echo')
+    if tube.recvline().startswith('/'):
+        tube.sendline('''exec python -c "import signal, os; signal.alarm(0); os.execl('$SHELL','')"''')
+        return tube
+
+    tube.sendline('which perl || echo')
+    if tube.recvline().startswith('/'):
+        tube.sendline('''exec perl -e "alarm 0; exec '${SHELL:-/bin/sh}'"''')
+        return tube
+
+    return None
 
 def register_sizes(regs, in_sizes):
     """Create dictionaries over register sizes and relations
 
     Given a list of lists of overlapping register names (e.g. ['eax','ax','al','ah']) and a list of input sizes,
     it returns the following:
-      * all_regs    : list of all valid registers
-      * sizes[reg]  : the size of reg in bits
-      * bigger[reg] : list of overlapping registers bigger than reg
-      * smaller[reg]: list of overlapping registers smaller than reg
+
+    * all_regs    : list of all valid registers
+    * sizes[reg]  : the size of reg in bits
+    * bigger[reg] : list of overlapping registers bigger than reg
+    * smaller[reg]: list of overlapping registers smaller than reg
 
     Used in i386/AMD64 shellcode, e.g. the mov-shellcode.
 
